<?php
class EJsWidgetBehavior extends CBehavior {

/**
 * Controller ID of the controller routes to the actions of the widget
 * Should be defined in the owner in order to be able to initialize the widget with that param
	public $widgetController;
**/

/**
 * Requested portlet action
 **/
	public $portletAction;
	
/**
 * for debugging
 **/
	public $forceDeployAssets = null;


/**
 * Utility
 **/
	protected static function normPath($path) {
		return preg_replace('#[\\\\/]+#', '/', $path);
	}
	
/**
 * 
 **/	
	protected static function getWidgetPath($class, $relPath='') {
		$r = new ReflectionClass($class);
		return self::normPath(dirname($r->getFileName()) . '/' . $relPath);
	}

/**
 * $class: class of the widget to get the js name to (can be a parent class of $o)
 * $o: widget object
 **/
	public static function getJsName($class, $o) {
		return strtolower($class) . '.js';
	}

/**
 * see above
 */
	public static function getCssName($class, $o) {
		return strtolower($class) . '.css';
	}
	
/**
 * Attributes/custom values to pass to the js widget object
 **/
	public function jsAttributes() {
		//key->value pairs or just keys, eg array('id','custom'=>123)
		return array(
			'id'
		);
	}

/**
 * Registers an asset file of the widget with its anchestors
 **/
	public function registerAssetsWithParents($class = false) {
		$o = $this->owner;
		if (!$class)
			$class = get_class($o);
		if ($class == 'CWidget') {
			Yii::app()->clientScript->registerCoreScript('jquery');
			return;
		}
		
		$o->registerAssetsWithParents(get_parent_class($class));
		
		$assetsDir = self::getWidgetPath($class) . '/assets/';
		if (file_exists($assetsDir)) {

			$assetsUrl = Yii::app()->assetManager->publish($assetsDir, false, -1, $o->forceDeployAssets);
			$cs = Yii::app()->clientScript;
			
			//registering js
			$method = 'getJsName';
			$path = method_exists($class, $method) ? $class::$method($class, $o) : self::$method($class, $o);
			if (file_exists($assetsDir . '/js/' . $path))
				$cs->registerScriptFile(self::normPath($assetsUrl . '/js/' . $path), CClientScript::POS_HEAD);

			//registering css
			$method = 'getCssName';
			$path = method_exists($class, $method) ? $class::$method($class, $o) : self::$method($class, $o);
			if (file_exists($assetsDir . '/css/' . $path))
				$cs->registerCssFile(self::normPath($assetsUrl . '/css/' . $path));
		}
	}


/**
 * Prepares the widget to be an "EJsWidget"
 * To be called from widget::init()
 * 
 * It also (by default, but optionally) registers the assets of the widget and creates the js instance for the widget
 **/
	public static function prepare($owner, $registerAssets = true, $createJsWidget = true) {
		if (!($owner instanceof CWidget) || isset($owner->__CLASS__))
		  return;

		$class = get_class($owner);


		$owner->attachBehavior(__CLASS__, array(
			'class'=>__CLASS__
		));

		$owner->portletAction = self::getPortletAction($class);
		
		if ($owner->portletAction) {
			if (!property_exists($class, 'widgetController')) {
				throw new Exception($class . ' must have attribute "widgetController"');
			}
			if (!$owner->widgetController)
				$owner->widgetController = $owner->controller->id;
		}		

		$owner->actionPrefix = strtolower($class) . '.';

		if ($owner->forceDeployAssets===null)
			$owner->forceDeployAssets = defined('YII_DEBUG') && YII_DEBUG;

		if ($registerAssets) {
			$owner->registerAssetsWithParents();
			if ($createJsWidget) {
				echo $owner->createJsWidget(true);
			}
		}
	}

/**
 * Generates the js code that creates the js instance
 **/
	public function createJsWidget($withScriptTags = true) {
		$o = $this->owner;
		
		$jsAttr = array(
			'id' => $o->id,
			'widgetController' => isset($o->widgetController) ? $o->widgetController : null,
			'baseUrl' => Yii::app()->request->baseUrl,
		);

		foreach($o->jsAttributes() as $key=>$val) {
			if (is_int($key)) {
				$key = $val;
				$val = $o->$val;
			}
				
			$jsAttr[$key] = $val;
		}
		
		$code = '$.ejswidget.create("' . get_class($o) . '", "' . $o->id . '", ' . CJSON::encode($jsAttr) . ');';
		if ($withScriptTags)
			$code = "<script>$code</script>";
		return $code;
	}

/**
 * Generates the code for the js object
 **/
	public function jsObj() {
		return '$.ejswidget.get("' . $this->owner->id . '")';
	}





/**
 * EXPERIMENTAL PART - supporting inline widget actions
**/


/**
 * Returns the requested inline widget action
 **/
	protected static function getPortletAction($class) {
		$m = array();
		if (!preg_match('#/' . strtolower($class) . '\\.(\\w+)$#', @$_REQUEST['r'], $m))
			return false;
		
		return @$m[1];	
	}


/**
 * Dirty hack: mocking CAction's usage inside widgets
 * Called from widget::runWithParams()
**/
	public static function runWithParams($_this, $params) {
		if (!$_this || !isset($params['r']))
			return false;
		$action = 'action' . ucfirst(preg_replace('#^.*\\.#', '', $params['r']));
		if (method_exists($_this, $action)) {
			$_this->init($action);//behavior assignment here
			$_this->$action();//running the action
			return true;
		}
		return false;
	}



/**
Publishes the actions of a widget.

To be called from within CWidget::actions() like this:

public static function actions() {
	return EJsWidgetBehavior::widgetActions(__CLASS__);
}
*/
	public static function widgetActions($class) {
		$action = self::getPortletAction($class);
		if (!$action)
			return array();

		$opt = @$_REQUEST["w"];
		if (empty($opt)) $opt = '{}';
		$opt = CJSON::decode($opt);
		
		//Removing possible non-member attributes
		foreach($opt as $p)
			if (!property_exists($class, $p))
				unset($opt[$p]);
		$opt['class'] = $class;

		return array(
			$action => $opt
		);
	}



/**
Routes to widget's actions from within a controller.

To be called from within CController->actions like this:

public function actions() {
	return EJsWidgetBehavior::controllerActions(array(
		'application.widgets.ecalculator.ECalculatorWidget',
		'application.widgets.efancycalculator.EFancyCalculatorWidget'
	));
}

Something like this will be generated:

return array(
	'ecalculatorwidget.' => array(
		'class' => 'application.widgets.ecalculator.ECalculatorWidget',
		'add'=>array(),
	),
	'efancycalculatorwidget.' => array(
		'class' => 'application.widgets.efancycalculator.EFancyCalculatorWidget',
		'add'=>array(),
		'mult'=>array()
	)
);

*/
	public static function controllerActions($aliases) {
		$actions = array();
		foreach($aliases as $alias) {
			$class = preg_replace('#^.*\\.#','',$alias);
			if (!class_exists($class,false))
				Yii::import($alias);
			$prefix = strtolower($class) . '.';
			$action = array(
				'class' => $class
			);
			foreach(get_class_methods($class) as $m)
				if (strpos($m, 'action')===0)
					$action[strtolower(substr($m,6))] = array();

			$actions[$prefix] = $action;
		}
		//Yii::log(print_r($actions,true));
		return $actions;
	}

}
